import psycopg2
import psycopg2.extras # For dictionary cursor
import logging
from .db_core import get_db_connection, get_domain # Import from new core module
from .image_utils import fetch_extract_save_image # Import the image function

from datetime import datetime, timedelta

# Configure basic logging for this module
logger = logging.getLogger(__name__)

def is_url_processed(url):
    """Checks if a URL has already been processed."""
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor() # No DictCursor needed for just checking existence
            cursor.execute("SELECT 1 FROM processed_urls WHERE url = %s", (url,))
            return cursor.fetchone() is not None
    except Exception as e:
        logger.error(f"Failed to check processed URL status for {url}: {e}", exc_info=True)
        return False # Assume not processed if error occurs

def add_processed_url(url):
    """Adds a URL to the processed_urls table."""
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            # Use ON CONFLICT DO NOTHING for PostgreSQL equivalent of INSERT OR IGNORE
            cursor.execute("INSERT INTO processed_urls (url) VALUES (%s) ON CONFLICT (url) DO NOTHING", (url,))
            return True
    except Exception as e:
        logger.error(f"Failed to add processed URL {url}: {e}", exc_info=True)
        return False

def save_final_article(title, summary, source_url, tags_list):
    """Saves a finalized news article to the database."""
    tags_str = ",".join(sorted(list(set(tags_list))))
    image_filename = None # Default to None
    article_id = None # We need the ID to name the image
    conn = None # Initialize conn to None

    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            # Insert first without image filename to get the ID
            # Use RETURNING id to get the new ID in PostgreSQL
            cursor.execute("""
                INSERT INTO final_articles (title, summary, source_url, tags)
                VALUES (%s, %s, %s, %s)
                RETURNING id
            """, (title, summary, source_url, tags_str))
            article_id = cursor.fetchone()[0] # Get the returned ID
            if article_id and source_url:
                 # Now try to fetch and save the image using the article_id
                 image_filename = fetch_extract_save_image(source_url, article_id)
                 if image_filename:
                     # Update the row with the image filename
                     cursor.execute("UPDATE final_articles SET image_filename = %s WHERE id = %s", (image_filename, article_id))

            # Commit happens automatically
            logger.info(f"Saved final article {article_id}: '{title}' (Image: {image_filename})")
            return True
    except Exception as e:
        logger.error(f"Failed to save final article '{title}': {e}", exc_info=True)
        return False

def record_scrape_failure(url: str, domain: str, error_type: str):
    """Records or updates a failed URL attempt in the database."""
    if not url or not domain:
        logger.warning(f"Attempted to record failure for invalid URL/domain: {url}")
        return False
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            # Check if URL already exists
            cursor.execute("SELECT id, failure_count FROM failed_urls WHERE url = %s", (url,))
            existing = cursor.fetchone()

            if existing:
                # Update existing entry
                new_count = existing['failure_count'] + 1
                cursor.execute("""
                    UPDATE failed_urls
                    SET failure_count = %s, last_attempt_timestamp = CURRENT_TIMESTAMP, error_type = %s
                    WHERE id = %s
                """, (new_count, error_type, existing['id']))
                logger.warning(f"Updated failed URL record (Count: {new_count}, Error: {error_type}): {url}")
            else:
                # Insert new entry
                cursor.execute("""
                    INSERT INTO failed_urls (url, domain_name, error_type)
                    VALUES (%s, %s, %s)
                """, (url, domain, error_type))
                logger.warning(f"Recorded new failed URL (Error: {error_type}): {url}")
            return True
    except Exception as e:
        logger.error(f"Failed to record scrape failure for {url} (Error: {error_type}): {e}", exc_info=True)
        return False

def is_domain_system_blacklisted(domain_name: str) -> bool:
    """Checks if a domain is in the system-wide blacklist."""
    if not domain_name:
        return False # Cannot blacklist an empty domain
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT 1 FROM system_domain_blacklist WHERE domain_name = %s", (domain_name.lower(),))
            return cursor.fetchone() is not None
    except Exception as e:
        logger.error(f"Failed to check system blacklist status for domain {domain_name}: {e}", exc_info=True)
        return False # Assume not blacklisted if error occurs

def add_domain_to_system_blacklist(domain_name: str, reason: str | None = None):
    """Adds a domain to the system blacklist."""
    clean_domain = get_domain(domain_name) # Ensure it's just the domain
    if not clean_domain:
        logger.warning(f"Attempted to system-blacklist invalid domain '{domain_name}'")
        return False
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("INSERT INTO system_domain_blacklist (domain_name, reason) VALUES (%s, %s) ON CONFLICT (domain_name) DO UPDATE SET reason = EXCLUDED.reason", (clean_domain, reason))
            logger.info(f"Added domain to system blacklist: {clean_domain} (Reason: {reason})")
            return True
    except Exception as e:
        logger.error(f"Failed to add domain '{clean_domain}' to system blacklist: {e}", exc_info=True)
        return False

def get_active_agent_configs() -> list[dict]:
    """
    Retrieves the configuration (id, name, tags, search_depth, is_enabled) for all agents
    that are enabled and have at least one subscriber.
    """
    active_agents = []
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            # Fetch agents that are enabled AND have at least one subscriber
            cursor.execute("""
                SELECT DISTINCT a.id, a.name, a.tags, a.search_depth, a.is_enabled
                FROM agents a
                JOIN user_subscriptions us ON a.id = us.agent_id
                WHERE a.is_enabled = TRUE
            """)
            rows = cursor.fetchall()
            active_agents = [dict(row) for row in rows]
    except Exception as e:
        logger.error(f"Failed to retrieve active agent configurations: {e}", exc_info=True)
    return active_agents

def get_recent_articles(hours: int = 48) -> list[dict]:
    """
    Retrieves articles published within the last N hours.

    Args:
        hours: The number of hours to look back. Defaults to 48.

    Returns:
        A list of dictionaries, each containing 'id', 'title', 'summary', 'created_timestamp'.
    """
    articles = []
    time_threshold = datetime.utcnow() - timedelta(hours=hours)
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute("""
                SELECT id, title, summary, created_timestamp
                FROM final_articles
                WHERE created_timestamp >= %s
            """, (time_threshold,))
            articles = [dict(row) for row in cursor.fetchall()]
    except Exception as e:
        logger.error(f"Failed to fetch recent articles (last {hours} hours): {e}", exc_info=True)
    return articles
